package java.util;

import java.io.ObjectInputStream;
import java.io.IOException;
import java.io.ObjectOutputStream;
import java.security.PermissionCollection;
import java.security.Permission;
import java.security.BasicPermission;

public final class PropertyPermission extends BasicPermission
{
    private static final int READ = 1;
    private static final int WRITE = 2;
    private static final int ALL = 3;
    private static final int NONE = 0;
    private transient int mask;
    private String actions;
    private static final long serialVersionUID = 885438825399942851L;
    
    private void init(int mask) {
        if ((mask & 0x3) != mask) {
            throw new IllegalArgumentException("invalid actions mask");
        }
        if (mask == 0) {
            throw new IllegalArgumentException("invalid actions mask");
        }
        if (this.getName() == null) {
            throw new NullPointerException("name can't be null");
        }
        this.mask = mask;
        return;
    }
    
    public PropertyPermission(String name, String actions) {
        super(name, actions);
        this.init(getMask(actions));
    }
    
    @Override
    public boolean implies(Permission permission) {
        if (!(permission instanceof PropertyPermission)) {
            return false;
        }
        PropertyPermission p = (PropertyPermission)permission;
        return (this.mask & p.mask) == p.mask && super.implies(p);
    }
    
    @Override
    public boolean equals(Object o) {
        if (o == this) {
            return true;
        }
        if (!(o instanceof PropertyPermission)) {
            return false;
        }
        PropertyPermission propertyPermission = (PropertyPermission)o;
        return this.mask == propertyPermission.mask && this.getName().equals(propertyPermission.getName());
    }
    
    @Override
    public int hashCode() {
        return this.getName().hashCode();
    }
    
    private static int getMask(String s) {
        int n = 0;
        if (s == null) {
            return n;
        }
        if (s == "read") {
            return 1;
        }
        if (s == "write") {
            return 2;
        }
        if (s == "read,write") {
            return 3;
        }
        char[] charArray = s.toCharArray();
        int i = charArray.length - 1;
        if (i < 0) {
            return n;
        }
        while (i != -1) {
            int n2;
            while (i != -1 && ((n2 = charArray[i]) == ' ' || n2 == '\r' || n2 == '\n' || n2 == '\f' || n2 == '\t')) {
                --i;
            }
            int n3;
            if (i >= 3 && (charArray[i - 3] == 'r' || charArray[i - 3] == 'R') && (charArray[i - 2] == 'e' || charArray[i - 2] == 'E') && (charArray[i - 1] == 'a' || charArray[i - 1] == 'A') && (charArray[i] == 'd' || charArray[i] == 'D')) {
                n3 = 4;
                n |= 0x1;
            }
            else if (i >= 4 && (charArray[i - 4] == 'w' || charArray[i - 4] == 'W') && (charArray[i - 3] == 'r' || charArray[i - 3] == 'R') && (charArray[i - 2] == 'i' || charArray[i - 2] == 'I') && (charArray[i - 1] == 't' || charArray[i - 1] == 'T') && (charArray[i] == 'e' || charArray[i] == 'E')) {
                n3 = 5;
                n |= 0x2;
            }
            else {
                throw new IllegalArgumentException("invalid permission: " + s);
            }
            for (int n4 = 0; i >= n3 && n4 == 0; --i) {
                switch (charArray[i - n3]) {
                    case ',': {
                        n4 = 1;
                        break;
                    }
                    case '\t':
                    case '\n':
                    case '\f':
                    case '\r':
                    case ' ': {
                        break;
                    }
                    default: {
                        throw new IllegalArgumentException("invalid permission: " + s);
                    }
                }
            }
            i -= n3;
        }
        return n;
    }
    
    static String getActions(int n) {
        StringBuilder sb = new StringBuilder();
        boolean n2 = false;
        if ((n & 0x1) == 0x1) {
            n2 = true;
            sb.append("read");
        }
        if ((n & 0x2) == 0x2) {
            if (!(n2 == 0)) {
                sb.append(',');
            }
            sb.append("write");
        }
        return sb.toString();
    }
    
    @Override
    public String getActions() {
        if (this.actions == null) {
            this.actions = getActions(this.mask);
        }
        return this.actions;
    }
    
    int getMask() {
        return this.mask;
    }
    
    @Override
    public PermissionCollection newPermissionCollection() {
        return new PropertyPermissionCollection();
    }
    
    private synchronized void writeObject(ObjectOutputStream objectOutputStream) throws IOException {
        if (this.actions == null) {
            this.getActions();
        }
        objectOutputStream.defaultWriteObject();
    }
    
    private synchronized void readObject(ObjectInputStream objectInputStream) throws IOException, ClassNotFoundException {
        objectInputStream.defaultReadObject();
        this.init(getMask(this.actions));
    }
}
